# -*- coding: utf-8 -*-

import os
import shutil
from pathlib import Path

import click
from git import Repo

from utils.Logger import get_logger
from utils.Utils import (
    get_bench_config_file,
    get_nuc_ip,
    set_bench_name,
    update_bench_config,
)


logger = get_logger("cli")
# BENCH_MAP_PATH = Path(__file__).resolve().parents[0] / "configs/bench_map.json"
BASE_DIR = Path(__file__).resolve().parents[0] / "bench_config"


@click.group()
def cli():
    pass


@click.command()
@click.option("--ip", default=None, help="This parameter is optional. default to automatic get your env ip and You " "can special your ip .")
@click.option("--file", default=None, help="This parameter is optional. You can special bench config to update .")
@click.option("--name", default=None, help="This parameter is optional. You can set bench name .")
@click.option("--submit", default=None, help="This parameter is optional. You can push your env config to cloud .")
def config(ip, file, submit, name):
    if not ip:
        ip = get_nuc_ip()
    if file and submit and submit.lower() == "true":
        update_bench_config(ip, file)
        push(ip)
    if file and (not submit or submit.lower() != "true"):
        update_bench_config(ip, file)
    if name:
        set_bench_name(ip, name)


def push(ip):
    # 获取配置文件路径
    if os.path.exists("/tmp/dm"):
        shutil.rmtree("/tmp/dm")
        logger.info("dm dir.")
    config_path = get_bench_config_file(ip, BASE_DIR)
    logger.info(f"local config file path -> {config_path}")
    giturl = "https://wiliam.wang:1234567890Dd@githost.nevint.com/niotest/automation/middleware/devicemanager.git"
    repo = Repo.clone_from(giturl, "/tmp/dm", branch="develop")
    path = config_path.split("/dm/configs")[1]
    destination = "/tmp/dm/dm/configs" + path
    logger.info(f"move config file  -> {destination}")
    if os.path.exists(destination):
        os.remove(destination)
    shutil.copy(config_path, destination)

    repo.index.add([destination])
    repo.index.commit(f"update config file: {path}")
    repo.remote().push()
    logger.info("submit config file success.")
    shutil.rmtree("/tmp/dm")
    logger.info("clean dm dir.")


cli.add_command(config)


def main():
    cli()
