"""
Author: chong.ma
Date: 2024-04-12 17:39:02
LastEditTime: 2024-04-12 17:39:25
FilePath: /root/bench_tool/bench_tool/net-relay.py
Description:

"""
import argparse
import re
import sys
import time

from pymodbus.client import ModbusTcpClient

from utils.ConfigManagers import DeviceManager
from utils.RelayManagers import RelayBoardDriver

USAGE = """
用于控制网络继电器板卡
1. 控制CCC/ZONE的上下电，需要在config中配置好映射关系
2. 控制单独某个口的上下电
python3 net-relay.py ALL RESET
python3 net-relay.py 2~3 RESET
python3 net-relay.py CCC RESET
python3 net-relay.py FZONE OFF
python3 net-relay.py LZONE OFF
python3 net-relay.py RZONE OFF

"""

if __name__ == "__main__":
    dm = DeviceManager()
    parser = argparse.ArgumentParser(usage=USAGE)

    def validate_arg1_format(value):
        # 检查是否为单个数字0到7
        if re.match(r"^[0-7]$", value):
            return int(value)
        # 检查是否为数字范围，如"3~5"
        elif re.match(r"^[0-7]~[0-7]$", value):
            return list(map(int, value.split("~")))
        elif value in ["ALL", "CCC", "FZONE", "RZONE", "LZONE"]:
            return value
        else:
            raise argparse.ArgumentTypeError("arg1 必须是0 1 2 or 3~5, ALL, CCC, FZONE, RZONE, LZONE")

    def validate_arg2_format(value):
        if value not in ("ON", "OFF", "RESET"):
            raise argparse.ArgumentTypeError("arg2 必须是 'ON', 'OFF', 'RESET'")
        return value

    parser.add_argument("arg1", type=validate_arg1_format, help="channel of board, eg.0 1 2 or 0~7, ALL")
    parser.add_argument("arg2", type=validate_arg2_format, help="status of board, eg.ON/OFF/RESET")
    args = parser.parse_args()

    if dm.BENCH_DICT.get("relay_type") == "net-relay":
        relay_dict = dm.BENCH_DICT.get("relays")
        relay_ip = dm.BENCH_DICT.get("relay_ip")
        with RelayBoardDriver(ip=relay_ip) as board:
            if isinstance(args.arg1, list):
                board.deal_all(args.arg2, args.arg1[0], args.arg1[1] + 1)
            elif isinstance(args.arg1, int):
                board.channel_ctrl(args.arg2, args.arg1)
            elif args.arg1 == "ALL":
                board.deal_all(args.arg2)
            elif args.arg1 == "CCC":
                btn = relay_dict.get("ccc_power_btn")
            elif args.arg1 == "FZONE":
                btn = relay_dict.get("f_zone_power_btn")
            elif args.arg1 == "RZONE":
                btn = relay_dict.get("r_zone_power_btn")
            elif args.arg1 == "LZONE":
                btn = relay_dict.get("l_zone_power_btn")
            else:
                print("Invalid arg1")

            if args.arg1 in ["CCC", "FZONE", "RZONE", "LZONE"]:
                if re.match(r"^[0-7]$", btn):
                    num = int(btn)
                    board.channel_ctrl(args.arg2, num)
                elif re.match(r"^[0-7]~[0-7]$", btn):
                    list = list(map(int, btn.split("~")))
                    board.deal_all(args.arg2, list[0], list[1] + 1)
