"""
Author: chong.ma
Date: 2024-03-28 15:54:13
LastEditTime: 2024-04-09 20:23:10
FilePath: /root/bench_tool/bench_tool/program_power.py
Description:

"""
import argparse
import os
import pprint as pp
import sys
import time

from utils.ConfigManagers import DeviceManager
from utils.PowerManagers import VISAInstrument

sys.path.append(os.path.dirname(os.path.dirname(__file__)))

USAGE = """

python3 program_power.py --whichone kikusui_1 --action LIST

"""

if __name__ == "__main__":
    dm = DeviceManager()
    parser = argparse.ArgumentParser(usage=USAGE)

    parser.add_argument("--whichone", help="whichone 必须是 kikusui_xxx/ngi_xxx", default="kikusui_1")
    parser.add_argument("--action", help="action 必须是 'GET', 'SET', 'ON', 'OFF', 'RESET', 'LIST'")
    parser.add_argument("--volt", help="volt 必须是 0~30V", type=float)
    parser.add_argument("--current", help="current 必须是 0~5A", type=float)
    args = parser.parse_args()
    power = dm.BENCH_DICT.get("powers").get(args.whichone)
    power_type = "ngi" if "ngi" in args.whichone else "kikusui"
    if args.action == "LIST":
        with VISAInstrument(power) as visaIns:
            visaIns.list_device()
    elif args.action == "GET":
        with VISAInstrument(power) as visaIns:
            pp.pprint(str(visaIns.read_actualVolt(power_type)) + " V")
            pp.pprint(str(visaIns.read_actualCurrent(power_type)) + " A")
    elif args.action == "SET":
        with VISAInstrument(power) as visaIns:
            visaIns.set_output(False, power_type)
            visaIns.set_current_value(args.current, power_type)
            visaIns.set_voltage_value(args.volt, power_type)
    elif args.action == "ON":
        with VISAInstrument(power) as visaIns:
            visaIns.set_output(True, power_type)
            pp.pprint(str(visaIns.read_output(power_type)))
    elif args.action == "OFF":
        with VISAInstrument(power) as visaIns:
            visaIns.set_output(False, power_type)
            pp.pprint(str(visaIns.read_output(power_type)))
    elif args.action == "RESET":
        with VISAInstrument(power) as visaIns:
            visaIns.set_output(False, power_type)
            time.sleep(3)
            visaIns.set_output(True, power_type)
